import 'package:meta/meta.dart';
import 'package:neon/src/platform/android.dart';
import 'package:neon/src/platform/linux.dart';
import 'package:neon/src/platform/platform.dart';
import 'package:path/path.dart' as p;
import 'package:path_provider/path_provider.dart';

/// Aurora specific platform information.
///
/// See:
///   * [NeonPlatform] to initialize and acquire an instance
///   * [AndroidNeonPlatform] for the Android implementation
///   * [LinuxNeonPlatform] for the Linux implementation
@immutable
@internal
class AuroraNeonPlatform implements NeonPlatform {
  /// Creates a new Aurora Neon platform.
  const AuroraNeonPlatform();

  @override
  bool get canUseCamera => true;

  @override
  bool get canUsePushNotifications => false;

  @override
  bool get canUseQuickActions => false;

  @override
  bool get canUseSystemTray => false;

  @override
  bool get canUseWebView => true;

  @override
  bool get canUseWindowManager => false;

  @override
  bool get canUseSharing => true;

  @override
  bool get shouldUseFileDialog => false;

  @override
  Future<String> get userAccessibleAppDataPath async {
    return p.join((await getDownloadsDirectory())!.path);
  }

  @override
  void init() {}
}
